@@ -4,7 +4,9 @@ import logging
 import sys
 import tempfile
 import traceback
+import warnings
 
+from asgiref.sync import async_to_sync, sync_to_async
 from django import http
 from django.conf import settings
 from django.core import signals
@@ -14,7 +16,6 @@ from django.http import FileResponse, HttpResponse, HttpResponseServerError
 from django.urls import set_script_prefix
 from django.utils.functional import cached_property
 
-from asgiref.sync import async_to_sync, sync_to_async
 from channels.exceptions import RequestAborted, RequestTimeout
 
 logger = logging.getLogger("django.request")
@@ -43,7 +44,8 @@ class AsgiRequest(http.HttpRequest):
         else:
             self.path_info = scope["path"]
 
-        # django path is different from asgi scope path args, it should combine with script name
+        # django path is different from asgi scope path args, it should combine
+        # with script name
         if self.script_name:
             self.path = "%s/%s" % (
                 self.script_name.rstrip("/"),
@@ -91,7 +93,8 @@ class AsgiRequest(http.HttpRequest):
                 corrected_name = "CONTENT_TYPE"
             else:
                 corrected_name = "HTTP_%s" % name.upper().replace("-", "_")
-            # HTTPbis say only ASCII chars are allowed in headers, but we latin1 just in case
+            # HTTPbis say only ASCII chars are allowed in headers, but we
+            # use latin1 just in case
             value = value.decode("latin1")
             if corrected_name in self.META:
                 value = self.META[corrected_name] + "," + value
@@ -166,22 +169,25 @@ class AsgiHandler(base.BaseHandler):
     # Size to chunk response bodies into for multiple response messages
     chunk_size = 512 * 1024
 
-    def __init__(self, scope):
+    def __init__(self):
+        msg = (
+            "Channel's inbuilt http protocal AsgiHandler is deprecated. "
+            "Use Django's get_asgi_application() instead."
+        )
+        warnings.warn(msg, DeprecationWarning, stacklevel=2)
+        super(AsgiHandler, self).__init__()
+        self.load_middleware()
+
+    async def __call__(self, scope, receive, send):
+        """
+        Async entrypoint - uses the sync_to_async wrapper to run things in a
+        threadpool.
+        """
         if scope["type"] != "http":
             raise ValueError(
                 "The AsgiHandler can only handle HTTP connections, not %s"
                 % scope["type"]
             )
-        super(AsgiHandler, self).__init__()
-        self.scope = scope
-        self.load_middleware()
-
-    async def __call__(self, receive, send):
-        """
-        Async entrypoint - uses the sync_to_async wrapper to run things in a
-        threadpool.
-        """
-        self.send = async_to_sync(send)
 
         # Receive the HTTP request body as a stream object.
         try:
@@ -189,7 +195,7 @@ class AsgiHandler(base.BaseHandler):
         except RequestAborted:
             return
         # Launch into body handling (and a synchronous subthread).
-        await self.handle(body_stream)
+        await self.handle(scope, async_to_sync(send), body_stream)
 
     async def read_body(self, receive):
         """Reads a HTTP body from an ASGI connection."""
@@ -212,19 +218,19 @@ class AsgiHandler(base.BaseHandler):
         return body_file
 
     @sync_to_async
-    def handle(self, body):
+    def handle(self, scope, send, body):
         """
         Synchronous message processing.
         """
         # Set script prefix from message root_path, turning None into empty string
-        script_prefix = self.scope.get("root_path", "") or ""
+        script_prefix = scope.get("root_path", "") or ""
         if settings.FORCE_SCRIPT_NAME:
             script_prefix = settings.FORCE_SCRIPT_NAME
         set_script_prefix(script_prefix)
-        signals.request_started.send(sender=self.__class__, scope=self.scope)
+        signals.request_started.send(sender=self.__class__, scope=scope)
         # Run request through view system
         try:
-            request = self.request_class(self.scope, body)
+            request = self.request_class(scope, body)
         except UnicodeDecodeError:
             logger.warning(
                 "Bad Request (UnicodeDecodeError)",
@@ -247,7 +253,7 @@ class AsgiHandler(base.BaseHandler):
                 response.block_size = 1024 * 512
         # Transform response into messages, which we yield back to caller
         for response_message in self.encode_response(response):
-            self.send(response_message)
+            send(response_message)
         # Close the response now we're done with it
         response.close()
 
@@ -304,13 +310,13 @@ class AsgiHandler(base.BaseHandler):
         response_headers = []
         for header, value in response.items():
             if isinstance(header, str):
-                header = header.encode("ascii")
+                header = header.encode("latin")
             if isinstance(value, str):
                 value = value.encode("latin1")
             response_headers.append((bytes(header), bytes(value)))
         for c in response.cookies.values():
             response_headers.append(
-                (b"Set-Cookie", c.output(header="").encode("ascii").strip())
+                (b"Set-Cookie", c.output(header="").encode("latin1").strip())
             )
         # Make initial response message
         yield {
