@@ -28,7 +28,7 @@ class CookieMiddleware:
     def __init__(self, inner):
         self.inner = inner
 
-    def __call__(self, scope):
+    async def __call__(self, scope, receive, send):
         # Check this actually has headers. They're a required scope key for HTTP and WS.
         if "headers" not in scope:
             raise ValueError(
@@ -38,13 +38,13 @@ class CookieMiddleware:
         # Go through headers to find the cookie one
         for name, value in scope.get("headers", []):
             if name == b"cookie":
-                cookies = parse_cookie(value.decode("ascii"))
+                cookies = parse_cookie(value.decode("latin1"))
                 break
         else:
             # No cookie header found - add an empty default.
             cookies = {}
         # Return inner application
-        return self.inner(dict(scope, cookies=cookies))
+        return await self.inner(dict(scope, cookies=cookies), receive, send)
 
     @classmethod
     def set_cookie(
@@ -121,13 +121,10 @@ class CookieMiddleware:
         )
 
 
-class SessionMiddleware:
+class InstanceSessionWrapper:
     """
-    Class that adds Django sessions (from HTTP cookies) to the
-    scope. Works with HTTP or WebSocket protocol types (or anything that
-    provides a "headers" entry in the scope).
-
-    Requires the CookieMiddleware to be higher up in the stack.
+    Populates the session in application instance scope, and wraps send to save
+    the session.
     """
 
     # Message types that trigger a session save if it's modified
@@ -136,51 +133,35 @@ class SessionMiddleware:
     # Message types that can carry session cookies back
     cookie_response_message_types = ["http.response.start"]
 
-    def __init__(self, inner):
-        self.inner = inner
+    def __init__(self, scope, send):
         self.cookie_name = settings.SESSION_COOKIE_NAME
         self.session_store = import_module(settings.SESSION_ENGINE).SessionStore
 
-    def __call__(self, scope):
-        return SessionMiddlewareInstance(scope, self)
-
-
-class SessionMiddlewareInstance:
-    """
-    Inner class that is instantiated once per scope.
-    """
-
-    def __init__(self, scope, middleware):
-        self.middleware = middleware
         self.scope = dict(scope)
+
         if "session" in self.scope:
-            # There's already session middleware of some kind above us, pass that through
+            # There's already session middleware of some kind above us, pass
+            # that through
             self.activated = False
         else:
             # Make sure there are cookies in the scope
             if "cookies" not in self.scope:
                 raise ValueError(
-                    "No cookies in scope - SessionMiddleware needs to run inside of CookieMiddleware."
+                    "No cookies in scope - SessionMiddleware needs to run "
+                    "inside of CookieMiddleware."
                 )
             # Parse the headers in the scope into cookies
             self.scope["session"] = LazyObject()
             self.activated = True
-        # Instantiate our inner application
-        self.inner = self.middleware.inner(self.scope)
 
-    async def __call__(self, receive, send):
-        """
-        We intercept the send() callable so we can do session saves and
-        add session cookie overrides to send back.
-        """
-        # Resolve the session now we can do it in a blocking way
-        session_key = self.scope["cookies"].get(self.middleware.cookie_name)
-        self.scope["session"]._wrapped = await database_sync_to_async(
-            self.middleware.session_store
-        )(session_key)
         # Override send
         self.real_send = send
-        return await self.inner(receive, self.send)
+
+    async def resolve_session(self):
+        session_key = self.scope["cookies"].get(self.cookie_name)
+        self.scope["session"]._wrapped = await database_sync_to_async(
+            self.session_store
+        )(session_key)
 
     async def send(self, message):
         """
@@ -194,14 +175,14 @@ class SessionMiddlewareInstance:
             # changed data, save it. We also save if it's empty as we might
             # not be able to send a cookie-delete along with this message.
             if (
-                message["type"] in self.middleware.save_message_types
+                message["type"] in self.save_message_types
                 and message.get("status", 200) != 500
                 and (modified or settings.SESSION_SAVE_EVERY_REQUEST)
             ):
                 await database_sync_to_async(self.save_session)()
                 # If this is a message type that can transport cookies back to the
                 # client, then do so.
-                if message["type"] in self.middleware.cookie_response_message_types:
+                if message["type"] in self.cookie_response_message_types:
                     if empty:
                         # Delete cookie if it's set
                         if settings.SESSION_COOKIE_NAME in self.scope["cookies"]:
@@ -223,7 +204,7 @@ class SessionMiddlewareInstance:
                         # Set the cookie
                         CookieMiddleware.set_cookie(
                             message,
-                            self.middleware.cookie_name,
+                            self.cookie_name,
                             self.scope["session"].session_key,
                             max_age=max_age,
                             expires=expires,
@@ -249,5 +230,30 @@ class SessionMiddlewareInstance:
             )
 
 
+class SessionMiddleware:
+    """
+    Class that adds Django sessions (from HTTP cookies) to the
+    scope. Works with HTTP or WebSocket protocol types (or anything that
+    provides a "headers" entry in the scope).
+
+    Requires the CookieMiddleware to be higher up in the stack.
+    """
+
+    def __init__(self, inner):
+        self.inner = inner
+
+    async def __call__(self, scope, receive, send):
+        """
+        Instantiate a session wrapper for this scope, resolve the session and
+        call the inner application.
+        """
+        wrapper = InstanceSessionWrapper(scope, send)
+
+        await wrapper.resolve_session()
+
+        return await self.inner(wrapper.scope, receive, wrapper.send)
+
+
 # Shortcut to include cookie middleware
-SessionMiddlewareStack = lambda inner: CookieMiddleware(SessionMiddleware(inner))
+def SessionMiddlewareStack(inner):
+    return CookieMiddleware(SessionMiddleware(inner))
