@@ -8,9 +8,11 @@ from channels.generic.websocket import (
     WebsocketConsumer,
 )
 from channels.layers import get_channel_layer
+from channels.sessions import SessionMiddlewareStack
 from channels.testing import WebsocketCommunicator
 
 
+@pytest.mark.django_db
 @pytest.mark.asyncio
 async def test_websocket_consumer():
     """
@@ -30,8 +32,10 @@ async def test_websocket_consumer():
         def disconnect(self, code):
             results["disconnected"] = code
 
+    app = TestConsumer()
+
     # Test a normal connection
-    communicator = WebsocketCommunicator(TestConsumer, "/testws/")
+    communicator = WebsocketCommunicator(app, "/testws/")
     connected, _ = await communicator.connect()
     assert connected
     assert "connected" in results
@@ -50,6 +54,48 @@ async def test_websocket_consumer():
     assert "disconnected" in results
 
 
+@pytest.mark.django_db
+@pytest.mark.asyncio
+async def test_multiple_websocket_consumers_with_sessions():
+    """
+    Tests that multiple consumers use the correct scope when using
+    SessionMiddleware.
+    """
+
+    class TestConsumer(WebsocketConsumer):
+        def connect(self):
+            self.accept()
+
+        def receive(self, text_data=None, bytes_data=None):
+            path = self.scope["path"]
+            self.send(text_data=path)
+
+    app = SessionMiddlewareStack(TestConsumer.as_asgi())
+
+    # Create to communicators.
+    communicator = WebsocketCommunicator(app, "/first/")
+    second_communicator = WebsocketCommunicator(app, "/second/")
+
+    connected, _ = await communicator.connect()
+    assert connected
+    connected, _ = await second_communicator.connect()
+    assert connected
+
+    # Test out of order
+    await second_communicator.send_to(text_data="Echo Path")
+    response = await second_communicator.receive_from()
+    assert response == "/second/"
+
+    await communicator.send_to(text_data="Echo Path")
+    response = await communicator.receive_from()
+    assert response == "/first/"
+
+    # Close out
+    await communicator.disconnect()
+    await second_communicator.disconnect()
+
+
+@pytest.mark.django_db
 @pytest.mark.asyncio
 async def test_websocket_consumer_subprotocol():
     """
@@ -61,15 +107,18 @@ async def test_websocket_consumer_subprotocol():
             assert self.scope["subprotocols"] == ["subprotocol1", "subprotocol2"]
             self.accept("subprotocol2")
 
+    app = TestConsumer()
+
     # Test a normal connection with subprotocols
     communicator = WebsocketCommunicator(
-        TestConsumer, "/testws/", subprotocols=["subprotocol1", "subprotocol2"]
+        app, "/testws/", subprotocols=["subprotocol1", "subprotocol2"]
     )
     connected, subprotocol = await communicator.connect()
     assert connected
     assert subprotocol == "subprotocol2"
 
 
+@pytest.mark.django_db
 @pytest.mark.asyncio
 async def test_websocket_consumer_groups():
     """
@@ -84,11 +133,13 @@ async def test_websocket_consumer_groups():
             results["received"] = (text_data, bytes_data)
             self.send(text_data=text_data, bytes_data=bytes_data)
 
+    app = TestConsumer()
+
     channel_layers_setting = {
         "default": {"BACKEND": "channels.layers.InMemoryChannelLayer"}
     }
     with override_settings(CHANNEL_LAYERS=channel_layers_setting):
-        communicator = WebsocketCommunicator(TestConsumer, "/testws/")
+        communicator = WebsocketCommunicator(app, "/testws/")
         await communicator.connect()
 
         channel_layer = get_channel_layer()
@@ -122,8 +173,10 @@ async def test_async_websocket_consumer():
         async def disconnect(self, code):
             results["disconnected"] = code
 
+    app = TestConsumer()
+
     # Test a normal connection
-    communicator = WebsocketCommunicator(TestConsumer, "/testws/")
+    communicator = WebsocketCommunicator(app, "/testws/")
     connected, _ = await communicator.connect()
     assert connected
     assert "connected" in results
@@ -153,9 +206,11 @@ async def test_async_websocket_consumer_subprotocol():
             assert self.scope["subprotocols"] == ["subprotocol1", "subprotocol2"]
             await self.accept("subprotocol2")
 
+    app = TestConsumer()
+
     # Test a normal connection with subprotocols
     communicator = WebsocketCommunicator(
-        TestConsumer, "/testws/", subprotocols=["subprotocol1", "subprotocol2"]
+        app, "/testws/", subprotocols=["subprotocol1", "subprotocol2"]
     )
     connected, subprotocol = await communicator.connect()
     assert connected
@@ -176,11 +231,13 @@ async def test_async_websocket_consumer_groups():
             results["received"] = (text_data, bytes_data)
             await self.send(text_data=text_data, bytes_data=bytes_data)
 
+    app = TestConsumer()
+
     channel_layers_setting = {
         "default": {"BACKEND": "channels.layers.InMemoryChannelLayer"}
     }
     with override_settings(CHANNEL_LAYERS=channel_layers_setting):
-        communicator = WebsocketCommunicator(TestConsumer, "/testws/")
+        communicator = WebsocketCommunicator(app, "/testws/")
         await communicator.connect()
 
         channel_layer = get_channel_layer()
@@ -210,16 +267,18 @@ async def test_async_websocket_consumer_specific_channel_layer():
             results["received"] = (text_data, bytes_data)
             await self.send(text_data=text_data, bytes_data=bytes_data)
 
+    app = TestConsumer()
+
     channel_layers_setting = {
         "testlayer": {"BACKEND": "channels.layers.InMemoryChannelLayer"}
     }
     with override_settings(CHANNEL_LAYERS=channel_layers_setting):
-        communicator = WebsocketCommunicator(TestConsumer, "/testws/")
+        communicator = WebsocketCommunicator(app, "/testws/")
         await communicator.connect()
 
         channel_layer = get_channel_layer("testlayer")
         # Test that the specific channel layer is retrieved
-        assert channel_layer != None
+        assert channel_layer is not None
 
         channel_name = list(channel_layer.channels.keys())[0]
         message = {"type": "websocket.receive", "text": "hello"}
@@ -231,6 +290,7 @@ async def test_async_websocket_consumer_specific_channel_layer():
         await communicator.disconnect()
 
 
+@pytest.mark.django_db
 @pytest.mark.asyncio
 async def test_json_websocket_consumer():
     """
@@ -246,8 +306,10 @@ async def test_json_websocket_consumer():
             results["received"] = data
             self.send_json(data)
 
+    app = TestConsumer()
+
     # Open a connection
-    communicator = WebsocketCommunicator(TestConsumer, "/testws/")
+    communicator = WebsocketCommunicator(app, "/testws/")
     connected, _ = await communicator.connect()
     assert connected
     # Test sending
@@ -276,8 +338,10 @@ async def test_async_json_websocket_consumer():
             results["received"] = data
             await self.send_json(data)
 
+    app = TestConsumer()
+
     # Open a connection
-    communicator = WebsocketCommunicator(TestConsumer, "/testws/")
+    communicator = WebsocketCommunicator(app, "/testws/")
     connected, _ = await communicator.connect()
     assert connected
     # Test sending
@@ -289,3 +353,74 @@ async def test_async_json_websocket_consumer():
     await communicator.send_to(bytes_data=b"w\0\0\0")
     with pytest.raises(ValueError):
         await communicator.wait()
+
+
+@pytest.mark.asyncio
+async def test_block_underscored_type_function_call():
+    """
+    Test that consumer prevent calling private functions as handler
+    """
+
+    class TestConsumer(AsyncWebsocketConsumer):
+        channel_layer_alias = "testlayer"
+
+        async def _my_private_handler(self, _):
+            await self.send(text_data="should never be called")
+
+    app = TestConsumer()
+
+    channel_layers_setting = {
+        "testlayer": {"BACKEND": "channels.layers.InMemoryChannelLayer"}
+    }
+    with override_settings(CHANNEL_LAYERS=channel_layers_setting):
+        communicator = WebsocketCommunicator(app, "/testws/")
+        await communicator.connect()
+
+        channel_layer = get_channel_layer("testlayer")
+        # Test that the specific channel layer is retrieved
+        assert channel_layer is not None
+
+        channel_name = list(channel_layer.channels.keys())[0]
+        # Should block call to private functions handler and raise ValueError
+        message = {"type": "_my_private_handler", "text": "hello"}
+        await channel_layer.send(channel_name, message)
+        with pytest.raises(
+            ValueError, match=r"Malformed type in message \(leading underscore\)"
+        ):
+            await communicator.receive_from()
+
+
+@pytest.mark.asyncio
+async def test_block_leading_dot_type_function_call():
+    """
+    Test that consumer prevent calling private functions as handler
+    """
+
+    class TestConsumer(AsyncWebsocketConsumer):
+        channel_layer_alias = "testlayer"
+
+        async def _my_private_handler(self, _):
+            await self.send(text_data="should never be called")
+
+    app = TestConsumer()
+
+    channel_layers_setting = {
+        "testlayer": {"BACKEND": "channels.layers.InMemoryChannelLayer"}
+    }
+    with override_settings(CHANNEL_LAYERS=channel_layers_setting):
+        communicator = WebsocketCommunicator(app, "/testws/")
+        await communicator.connect()
+
+        channel_layer = get_channel_layer("testlayer")
+        # Test that the specific channel layer is retrieved
+        assert channel_layer is not None
+
+        channel_name = list(channel_layer.channels.keys())[0]
+        # Should not replace dot by underscore and call private function (see
+        # issue: #1430)
+        message = {"type": ".my_private_handler", "text": "hello"}
+        await channel_layer.send(channel_name, message)
+        with pytest.raises(
+            ValueError, match=r"Malformed type in message \(leading underscore\)"
+        ):
+            await communicator.receive_from()
